@@ -1,5 +1,5 @@
 // ****************************************************************************
-// Prelude, this file is used with -macro_file_builtins option of the C parser
+// Prelude, this file is used with --macro-file-builtins option of the C parser
 // ****************************************************************************
 
 /* This file contains:
@@ -87,117 +87,121 @@
 // ----------------------------------------------------------------------------
 // Attributes. could perhaps generalize via "__.*"
 // ----------------------------------------------------------------------------
-#define  __init
-#define  __initconst
-#define  __page_aligned_data
-#define  __page_aligned_bss
-#define  __always_unused
-#define  __visible
-#define  __exit
-#define  __user
-#define  __iomem
-#define  __initdata
-#define  __exitdata
-#define  __devinit
-#define  __devexit
-#define  __devinitdata
-#define  __cpuinit
-#define  __cpuinitdata
-#define  __init_or_module
-#define  __initdata_or_module
-#define  __pminit
-#define  __pminitdata
-#define  __irq_entry
+#define  __init MACROANNOTATION
+#define  __initconst MACROANNOTATION
+#define  __page_aligned_data MACROANNOTATION
+#define  __page_aligned_bss MACROANNOTATION
+#define  __always_unused MACROANNOTATION
+#define  __visible MACROANNOTATION
+#define  __exit MACROANNOTATION
+#define  __user MACROANNOTATION
+#define  __iomem MACROANNOTATION
+#define  __initdata MACROANNOTATION
+#define  __exitdata MACROANNOTATION
+#define  __devinit MACROANNOTATION
+#define  __devexit MACROANNOTATION
+#define  __devinitdata MACROANNOTATION
+#define  __cpuinit MACROANNOTATION
+#define  __cpuinitdata MACROANNOTATION
+#define  __init_or_module MACROANNOTATION
+#define  __initdata_or_module MACROANNOTATION
+#define  __pminit MACROANNOTATION
+#define  __pminitdata MACROANNOTATION
+#define  __irq_entry MACROANNOTATION
+#define  __free MACROANNOTATION
 
-#define  __cacheline_aligned
-#define  ____cacheline_aligned
-#define  __cacheline_aligned_in_smp
-#define  ____cacheline_aligned_in_smp
-#define  ____cacheline_internodealigned_in_smp
+#define  __cacheline_aligned MACROANNOTATION
+#define  ____cacheline_aligned MACROANNOTATION
+#define  __cacheline_aligned_in_smp MACROANNOTATION
+#define  ____cacheline_aligned_in_smp MACROANNOTATION
+#define  ____cacheline_internodealigned_in_smp MACROANNOTATION
 
-#define  __ALIGNED__
-#define  __3xp_aligned
+#define  __ALIGNED__ MACROANNOTATION
+#define  __3xp_aligned MACROANNOTATION
 
-#define  __pmac
-#define  __force
-#define  __nocast
-#define  __read_mostly
+#define  __pmac MACROANNOTATION
+#define  __force MACROANNOTATION
+#define  __read_mostly MACROANNOTATION
 
-#define  __must_check
+#define  __must_check MACROANNOTATION
 // pb
-#define  __unused
-#define  __maybe_unused
+#define  __unused MACROANNOTATION
+#define  __maybe_unused MACROANNOTATION
 
 
-#define  __attribute_used__
-#define  __attribute_pure__
-#define  __attribute_const__
+#define  __attribute_used__ MACROANNOTATION
+#define  __attribute_pure__ MACROANNOTATION
+#define  __attribute_const__ MACROANNOTATION
 // #define _attribute__const __attribute__((const))
 
-#define  __always_inline
+#define  __always_inline MACROANNOTATION
 
-#define  __xipram
+#define  __xipram MACROANNOTATION
 
 // in the other part of the kernel, in arch/, mm/, etc
-#define  __sched
-#define  __initmv
-#define  __exception
-#define  __cpuexit
-#define  __kprobes
-#define  __meminit
-#define  __meminitdata
-#define  __nosavedata
-#define  __kernel
-#define  __nomods_init
-#define  __apicdebuginit
-#define  __ipc_init
-#define  __modinit
-#define  __lockfunc
-#define  __weak
-#define  __tlb_handler_align
-#define  __lock_aligned
-#define  __force_data
-#define  __nongprelbss
-#define  __nongpreldata
-#define  __noreturn
+#define  __sched MACROANNOTATION
+#define  __initmv MACROANNOTATION
+#define  __exception MACROANNOTATION
+#define  __cpuexit MACROANNOTATION
+#define  __kprobes MACROANNOTATION
+#define  __meminit MACROANNOTATION
+#define  __meminitdata MACROANNOTATION
+#define  __nosavedata MACROANNOTATION
+#define  __kernel MACROANNOTATION
+#define  __nomods_init MACROANNOTATION
+#define  __apicdebuginit MACROANNOTATION
+#define  __ipc_init MACROANNOTATION
+#define  __modinit MACROANNOTATION
+#define  __lockfunc MACROANNOTATION
+#define  __weak MACROANNOTATION
+#define  __tlb_handler_align MACROANNOTATION
+#define  __lock_aligned MACROANNOTATION
+#define  __force_data MACROANNOTATION
+#define  __nongprelbss MACROANNOTATION
+#define  __nongpreldata MACROANNOTATION
+#define  __noreturn MACROANNOTATION
+#define  __nocast MACROANNOTATION
 
-#define  __section_jiffies
-#define  __vsyscall_fn
-#define  __section_vgetcpu_mode
-#define  __section_vsyscall_gtod_data
+#define  __section_jiffies MACROANNOTATION
+#define  __vsyscall_fn MACROANNOTATION
+#define  __section_vgetcpu_mode MACROANNOTATION
+#define  __section_vsyscall_gtod_data MACROANNOTATION
+#define  __section(x) MACROANNOTATION
 
 // in header files
-#define  __bitwise
-#define  __bitwise__
-#define  __deprecated
+#define  __bitwise MACROANNOTATION
+#define  __bitwise__ MACROANNOTATION
+#define  __deprecated MACROANNOTATION
 
 
 // last found
-#define __init_refok
+#define __init_refok MACROANNOTATION
 
 
 // maybe only in old kernel
-#define  __openfirmware
+#define  __openfirmware MACROANNOTATION
 
 // now in lexer
 //#define __extension__
 
-#define __thread
-#define __used
-#define __pure
+#define __thread MACROANNOTATION
+#define __used MACROANNOTATION
+#define __pure MACROANNOTATION
 
-#define __ref
-#define __refdata
+#define __ref MACROANNOTATION
+#define __refdata MACROANNOTATION
 
-#define __uses_jump_to_uncached
+#define __uses_jump_to_uncached MACROANNOTATION
 
 
 // last last
-#define __net_init
-#define __net_exit
-#define __net_initdata
+#define __net_init MACROANNOTATION
+#define __net_exit MACROANNOTATION
+#define __net_initdata MACROANNOTATION
 
-#define __paginginit // in mm
+#define __paginginit MACROANNOTATION // in mm
+#define __ro_after_init MACROANNOTATION
+#define __lsm_ro_after_init MACROANNOTATION
 
 // ----------------------------------------------------------------------------
 // String macros
@@ -273,6 +277,9 @@
 
 #define  STATIC_INLINE static inline
 #define  __EXTERN_INLINE extern inline
+#define INLINE inline
+#define FORCE_INLINE_TEMPLATE inline
+#define nokprobe_inline inline
 
 #define  AGPEXTERN extern
 
@@ -298,7 +305,6 @@
 #define  fastcall
 #define  asmlinkage
 
-#define  far
 #define  SK_FAR
 
 // pb
@@ -321,9 +327,9 @@
 #define  ASC_INITDATA
 #define  in2000__INITDATA
 #define  PACKED
-#define  __packed
-#define  __rcu
-#define  __percpu
+#define  __packed MACROANNOTATION
+#define  __rcu MACROANNOTATION
+#define  __percpu MACROANNOTATION
 
 #define  WPMINFO
 #define  CPMINFO
@@ -341,11 +347,10 @@
 #define  nabi_no_regargs
 
 
-#define  ATTRIB_NORET
-#define  ATTRIBUTE_UNUSED
+#define  ATTRIB_NORET MACROANNOTATION
+#define  ATTRIBUTE_UNUSED MACROANNOTATION
 #define  BTEXT
 #define  BTDATA
-#define  PAGE_ALIGNED
 
 #define  EARLY_INIT_SECTION_ATTR
 
@@ -365,15 +370,20 @@
 // ----------------------------------------------------------------------------
 // ----------------------------------------------------------------------------
 
-#define __releases(x)
-#define __acquires(x)
+#define __releases(x) MACROANNOTATION
+#define __acquires(x) MACROANNOTATION
+
+#define __must_hold(x) MACROANNOTATION
+#define __printf(a,b) MACROANNOTATION
 
 //now in lexer
 //#define __declspec(x)
 
-#define __page_aligned(x)
-#define __aligned(x)
-#define __vsyscall(x)
+#define __page_aligned(x) MACROANNOTATION
+#define __aligned(x) MACROANNOTATION
+#define __vsyscall(x) MACROANNOTATION
+
+#define noinstr
 
 // ----------------------------------------------------------------------------
 // ----------------------------------------------------------------------------
@@ -414,26 +424,23 @@ static const struct machine_desc __mach_desc_##_type	\
 // include/asm-i386/pci.h
 // the DECLARE are detected by parsing_hack but not when they are
 // inside a struct def.
-#define DECLARE_PCI_UNMAP_ADDR(ADDR_NAME)
-#define DECLARE_PCI_UNMAP_LEN(LEN_NAME)
+#define DECLARE_PCI_UNMAP_ADDR YACFE_DECLARATOR
+#define DECLARE_PCI_UNMAP_LEN YACFE_DECLARATOR
 
 // defined in drivers/infiniband/hw/mthca/mthca_doorbell.h
-#define MTHCA_DECLARE_DOORBELL_LOCK(doorbell_lock)
+//#define MTHCA_DECLARE_DOORBELL_LOCK(doorbell_lock)
 
 // include/linux/types.h
 //#define BITS_TO_LONGS(bits) \
 //	(((bits)+BITS_PER_LONG-1)/BITS_PER_LONG)
-#define DECLARE_BITMAP(name,bits)
-//	/*unsigned*/ long name[BITS_TO_LONGS(bits)]
-
-
-// include/asm-i386/percpu.h
-// interesting macro where we see the need of __typeof__(type) with
-// for example DECLARE_PER_CPU(char[256], iucv_dbf_txt_buf);
-#define DEFINE_PER_CPU(type, name) \
-    __attribute__((__section__(".data.percpu"))) __typeof__(type) per_cpu__##name
-#define DECLARE_PER_CPU(type, name) extern __typeof__(type) per_cpu__##name
+#define DECLARE_BITMAP(x) YACFE_DECLARATOR
+#define DEFINE_PER_CPU(x) YACFE_DECLARATOR
+#define DECLARE_PER_CPU(x) YACFE_DECLARATOR
 
+#define DEFINE_STATIC_KEY_FALSE(x) YACFE_DECLARATOR
+#define DEFINE_STATIC_KEY_FALSE_RO(x) YACFE_DECLARATOR
+#define DEFINE_STATIC_KEY_TRUE(x) YACFE_DECLARATOR
+#define DEFINE_STATIC_KEY_TRUE_RO(x) YACFE_DECLARATOR
 
 
 // include/linux/kobject.h
@@ -764,6 +771,8 @@ do {									\
 // system calls
 // ----------------------------------------------------------------------------
 
+#define SYSCALL_DEFINE0(func) \
+    asmlinkage unsigned long func(void)
 #define SYSCALL_DEFINE1(func, t1, a1) \
     asmlinkage unsigned long func(t1 a1)
 #define SYSCALL_DEFINE2(func, t1, a1, t2, a2) \
@@ -777,4 +786,45 @@ do {									\
 #define SYSCALL_DEFINE6(func, t1, a1, t2, a2, t3, a3, t4, a4, t5, a5, t6, a6) \
     asmlinkage unsigned long func(t1 a1, t2 a2, t3 a3, t4 a4, t5 a5, t6 a6)
 
+#define COMPAT_SYSCALL_DEFINE0(func) \
+    asmlinkage unsigned long func(void)
+#define COMPAT_SYSCALL_DEFINE1(func, t1, a1) \
+    asmlinkage unsigned long func(t1 a1)
+#define COMPAT_SYSCALL_DEFINE2(func, t1, a1, t2, a2) \
+    asmlinkage unsigned long func(t1 a1, t2 a2)
+#define COMPAT_SYSCALL_DEFINE3(func, t1, a1, t2, a2, t3, a3) \
+    asmlinkage unsigned long func(t1 a1, t2 a2, t3 a3)
+#define COMPAT_SYSCALL_DEFINE4(func, t1, a1, t2, a2, t3, a3, t4, a4) \
+    asmlinkage unsigned long func(t1 a1, t2 a2, t3 a3, t4 a4)
+#define COMPAT_SYSCALL_DEFINE5(func, t1, a1, t2, a2, t3, a3, t4, a4, t5, a5) \
+    asmlinkage unsigned long func(t1 a1, t2 a2, t3 a3, t4 a4, t5 a5)
+#define COMPAT_SYSCALL_DEFINE6(func, t1, a1, t2, a2, t3, a3, t4, a4, t5, a5, t6, a6) \
+    asmlinkage unsigned long func(t1 a1, t2 a2, t3 a3, t4 a4, t5 a5, t6 a6)
+
+#define BPF_CALL_0(func) \
+    u64 func(void)
+#define BPF_CALL_1(func, t1, a1) \
+    u64 func(t1 a1)
+#define BPF_CALL_2(func, t1, a1, t2, a2) \
+    u64 func(t1 a1, t2 a2)
+#define BPF_CALL_3(func, t1, a1, t2, a2, t3, a3) \
+    u64 func(t1 a1, t2 a2, t3 a3)
+#define BPF_CALL_4(func, t1, a1, t2, a2, t3, a3, t4, a4) \
+    u64 func(t1 a1, t2 a2, t3 a3, t4 a4)
+#define BPF_CALL_5(func, t1, a1, t2, a2, t3, a3, t4, a4, t5, a5) \
+    u64 func(t1 a1, t2 a2, t3 a3, t4 a4, t5 a5)
+
 #define KBUILD_MODNAME YACFE_STRING
+
+// Macros for automatic memory management from GLib
+// https://developer-old.gnome.org/glib/stable/glib-Miscellaneous-Macros.html#g-autoptr
+#define g_autoptr(TypeName) TypeName*
+#define g_autolist(TypeName) GList*
+#define g_autoslist(TypeName) GSList*
+#define g_autoqueue(TypeName) GQueue*
+#define g_auto(TypeName) TypeName
+#define g_autofree
+
+// found in task_struct
+#define randomized_struct_fields_start YACFE_DECLARATOR
+#define randomized_struct_fields_end YACFE_DECLARATOR
